# -*-: coding utf-8 -*-
""" Sonos skill for Snips. """

import datetime
import soco

from soco.music_services import MusicService
from soco.alarms import Alarm
from random import shuffle
from sonos_radio.handler.spotify.spotify import SpotifyClient

MAX_VOLUME = 70
GAIN = 4
ALARM_DURATION = datetime.time(second=20) # in secondes

MAP_TO_INT = {
    None: 1,
    'un': 1,
    'deux': 2,
    'trois': 3,
    'quatre': 4,
    'cinq': 5,
    'six': 6,
    'sept': 7,
    'huit': 8,
    'neuf': 9,
    'dix': 10
}

class SnipsSonos:
    """ Sonos skill for Snips. """
    
    def __init__(self, spotify_client_id, spotify_client_secret, spotify_refresh_token):
        # find the device
        self.device = list(soco.discover())[0]
        self.tunein = MusicService('TuneIn')
        self.max_volume = MAX_VOLUME
        self.spotify = SpotifyClient(spotify_client_id, spotify_client_secret, spotify_refresh_token)

    def pause_sonos(self):
        self.device.pause()

    def volume_up(self, level):
        level = MAP_TO_INT[level]
        current_volume = self.device.volume
        self.device.volume = min(
            current_volume + GAIN * level,
            self.max_volume)
        self.device.play()

    def volume_down(self, level):
        level = MAP_TO_INT[level]
        self.device.volume -= GAIN * level
        self.device.play()
        print self.device.volume

    def set_volume(self, volume_value):
        self.device.volume = volume_value
        self.device.play()

    def stop_sonos(self):
        self.device.stop()

    def turn_on_radio(self, radio_name):
        res = self.tunein.search('stations', term=radio_name)
        if 'mediaMetadata' not in res:
            return "radio not found"
        if isinstance(res['mediaMetadata'], list):
            radio_id = res['mediaMetadata'][0]['id']
        elif isinstance(res['mediaMetadata'], dict):
            radio_id = res['mediaMetadata']['id']
        else:
            raise TypeError("Unknown type for tune in search metadata")
        radio_uri = self.tunein.get_media_uri(radio_id)
        # queue = self.device.get_queue()
        self.device.play_uri(radio_uri.replace('http', 'x-rincon-mp3radio'))

    # def turn_on_playlist(self, playlist_name):
    #     res = self.spotify.search('playlists', term=playlist_name)
    #     if 'mediaMetadata' not in res:
    #         return "playlist not found"
    #     if isinstance(res['mediaMetadata'], list):
    #         playlist_id = res['mediaMetadata'][0]['id']
    #     elif isinstance(res['mediaMetadata'], dict):
    #         playlist_id = res['mediaMetadata']['id']
    #     else:
    #         raise TypeError("Unknown type for spotify search metadata")
    #     playlist_uri = self.tunein.get_media_uri(playlist_id)
    #     self.device.play_uri(playlist_uri.replace('http', 'x-rincon-mp3radio'))

    def set_alarm(self, alarm_time):
        n_seconds = alarm_time['hours'] * 3600 + \
                    alarm_time['minutes'] * 60 + \
                    alarm_time['seconds']
        print ALARM_DURATION
        alarm = Alarm(
            self.device,
            start_time=datetime.datetime.now() + datetime.timedelta(
                seconds=n_seconds),
            duration=ALARM_DURATION,
            recurrence="ONCE")
        alarm.save()

    def format_uri_spotify(self, uri):
        return 'x-sonos-spotify:' + uri.replace(':', '%3a') + \
            "?sid=9"

    def play_playlist(self, name, _shuffle=False):
        tracks = self.spotify.get_tracks_from_playlist(name)
        if tracks is None:
            return None
        self.device.stop()
        self.device.clear_queue()
        if _shuffle:
            shuffle(tracks)
        # self.device.play_uri(
        #     self.format_uri_spotify(tracks[0]['track']['uri']))
        for track in tracks:
            self.device.add_uri_to_queue(
                self.format_uri_spotify(track['track']['uri'])
            )
        self.device.play_from_queue(0)
        # queue = self.device.get_queue()
        # self.device.play_uri(
        #     "x-sonos-spotify:" + self.spotify.user_playlists[name]['uri']
        # )

    def play_artist(self, name):
        tracks = self.spotify.get_top_tracks_from_artist(name)
        if tracks is None:
            return None
        self.device.stop()
        self.device.clear_queue()
        for track in tracks:
            self.device.add_uri_to_queue(
                self.format_uri_spotify(track['uri'])
            )
        self.device.play_from_queue(0)

    def play_next_item_in_queue(self):
        try:
            self.device.next()
        except Exception:
            print "Failed to play next item, maybe last song?"

    def play_previous_item_in_queue(self):
        try:
            self.device.previous()
        except Exception:
            print "Failed to play previous item, maybe first song?"



# -*-: coding utf-8 -*-
""" Sonos skill for Snips. """

import codecs
import os
import json
import requests

class SpotifyClient():

    def __init__(self, spotify_client_id, spotify_client_secret, spotify_refresh_token):
        self.client_id = spotify_client_id
        self.client_secret = spotify_client_secret
        self.refresh_token = spotify_refresh_token
        self.get_user_playlists()
        self.get_user_id()

    def refresh_access_token(self):
        _r = requests.post(
            "https://accounts.spotify.com/api/token",
            data={
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "grant_type": "refresh_token",
                "refresh_token": self.refresh_token
            })
        self.access_token = _r.json()['access_token']

    def get_user_id(self):
        _r = requests.get(
            "https://api.spotify.com/v1/me",
            headers={
                "Authorization": "Bearer {}".format(self.access_token),
            })
        self.user_id = _r.json()['id']

    def get_user_playlists(self):
        # TODO: get all playlists if there are more than 50 by looping
        # and using the offset parameters
        self.refresh_access_token()
        _r = requests.get(
            "https://api.spotify.com/v1/me/playlists",
            params={
                'limit': 50
            },
            headers={
                "Authorization": "Bearer {}".format(self.access_token),
            })
        self.user_playlists = {
            playlist['name'].lower(): playlist for
            playlist in _r.json()['items']}

    def get_tracks_from_playlist(self, name):
        self.refresh_access_token()
        try:
            _r = requests.get(
                self.user_playlists[name]['tracks']['href'],
                params={
                    'limit': 100
                },
                headers={
                    "Authorization": "Bearer {}".format(self.access_token),
                })
        except KeyError:
            print "Unknown playlist {}".format(name)
            return None
        return _r.json()['items']

    def _dump_favorite_artists(self, n_artists, output_name):
        self.refresh_access_token()
        all_artists_names = []
        n_found_artists = 0
        while n_found_artists < n_artists:
            _r = requests.get(
                'https://api.spotify.com/v1/me/top/artists',
                params={
                    'limit': min(50, n_artists - n_found_artists),
                    # 50 is the maximum
                    'offset': n_found_artists
                },
                headers={
                    "Authorization": "Bearer {}".format(self.access_token),
                }
            )
            all_artists_names.extend(
                [item['name'] for item in _r.json()['items']])
            if len(all_artists_names) == n_found_artists:
                print "Maximum number of artists reached: {}".format(
                    n_found_artists)
                break
            n_found_artists = len(all_artists_names)
        with codecs.open(output_name, 'w', 'utf-8') as f:
            f.write(u"\n".join(all_artists_names))

    def get_top_tracks_from_artist(self, artist):
        self.refresh_access_token()
        # First get artist id
        try:
            _r = requests.get(
                'https://api.spotify.com/v1/search',
                params={
                    'q': artist,
                    'type': 'artist'
                },
                headers={
                    "Authorization": "Bearer {}".format(self.access_token)
                }
            )
            _id = _r.json()['artists']['items'][0]['id']
            # Get list of top tracks from artist
            _r = requests.get(
                'https://api.spotify.com/v1/artists/{}/top-tracks'.format(_id),
                params={
                    'country': 'fr'
                },
                headers={
                    "Authorization": "Bearer {}".format(self.access_token)
                }
            )
            return _r.json()['tracks']
        except Exception:
            return None



from setuptools import setup

setup(
    name='snipssonos',
    version='1.0.0',
    description='Sonos skill for Snips',
    author='The Als',
    url='https://github.com/snipsco/snips-skill-sonos',
    download_url='',
    license='MIT',
    install_requires=['soco'],
    test_suite="tests",
    keywords=['snips', 'sonos'],
    packages=['snipssonos'],
    package_data={'snipssonos': ['Snipsspec']},
    include_package_data=True,
)
